using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Lightweight_Subwindow(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {

            /* Sample: Lightweight_Subwindow
            * This sample will show you how to:
            * - Load an HSF file through Stream and check for Stream IO exceptions
            * - Create a legend with a dedicated camera
            * - Use a lightweight sub-window to keeps geometry on top of the scene 
            * 
            * Read more about the topics covered in this sample in our Programming Guide */

            HPS.SegmentKey driveShaft = modelKey.Subsegment();
            HPS.SegmentKey legendBar = modelKey.Subsegment();


            /* This next block of code shows you how to load an HSF file
	         * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
	         * 2. Use the Stream::File::Import method to load the desired file, with the options you specified
	         * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	         * 4. Use a try-catch block to capture any Stream IO exceptions */
            HPS.Stream.ImportNotifier notifier;
	        try
	        {
                HPS.Stream.ImportOptionsKit importOptionsKit = new HPS.Stream.ImportOptionsKit();
		        importOptionsKit.SetSegment(driveShaft);
		        string filename = INPUT_FOLDER + "/driveshaft.hsf";

                notifier = HPS.Stream.File.Import(filename, importOptionsKit);
		        notifier.Wait();

		        importOptionsKit.SetSegment(legendBar);
		        filename = INPUT_FOLDER + "/legend.hsf";

                notifier = HPS.Stream.File.Import(filename, importOptionsKit);
		        notifier.Wait();
	        }
            catch (HPS.IOException)
	        {
		        /* catch Stream I/O Errors here */
	        }

            /* The next block of code shows you how to set up a new Camera from a CameraKit object
	         * 1. Set up the camera position, target, up vector, field and projection attributes
	         * 2. Apply the camera kit to the segment where the camera needs to be inserted */
            HPS.CameraKit camera = new HPS.CameraKit();
            camera.SetPosition(new HPS.Point(0.35f, 0.48f, 0.23f)).SetTarget(new HPS.Point(0, 0, 0)).SetUpVector(new HPS.Vector(-0.63f, 0.64f, -0.43f)).
                SetField(0.18f, 0.18f).SetProjection(HPS.Camera.Projection.Orthographic);
	        driveShaft.SetCamera(camera);

            camera.SetPosition(new HPS.Point(0, 0, 1)).SetTarget(new HPS.Point(0.3f, 0.3f, 0)).SetUpVector(new HPS.Vector(0, 1, 0)).
                SetField(0.4f, 0.4f).SetProjection(HPS.Camera.Projection.Orthographic);
	        legendBar.SetCamera(camera);

            /* To make sure that the legend will be displayed on top of the geometry, it is placed in a 
	         * lightweight window. */
            legendBar.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, -1, 1), HPS.Subwindow.Type.Lightweight);
            legendBar.GetTextAttributeControl().SetSize(0.015f, HPS.Text.SizeUnits.SubscreenRelative)
                .SetAlignment(HPS.Text.Alignment.BottomLeft);

            view.FitWorld().Update();
        }
    }
}
