using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Reference_Geometry(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {

	        // We create a root segment that becomes the include library. It is often useful
	        // but not necessary to organize your instanced geometry into a library.
            HPS.SegmentKey rootSegment = HPS.Database.CreateRootSegment();
            HPS.SegmentKey shellSegment = rootSegment.Subsegment("include library").Subsegment("shell");

	        // Here we are defining the shell that will be instanced. This will be a pyramid.
	        HPS.Point[] pointArray = new HPS.Point[5];
            pointArray[0] = new HPS.Point(0, 0, 0);
	        pointArray[1] = new HPS.Point(1, 0, 0);
            pointArray[2] = new HPS.Point(1, 0, 1);
            pointArray[3] = new HPS.Point(0, 0, 1);
            pointArray[4] = new HPS.Point(0.5f, 0.5f, 0.75f);

	        int[] faceList = { 4, 0, 1, 2, 3, 
	                           3, 4, 0, 1,
	                           3, 4, 1, 2,
	                           3, 4, 2, 3, 
	                           3, 4, 3, 0 };

            HPS.ShellKit shellKit = new HPS.ShellKit();
	        shellKit.SetPoints(pointArray);
	        shellKit.SetFacelist(faceList);
	
	        // Insert the shell that was defined above.
            HPS.ShellKey myShellKey = shellSegment.InsertShell(shellKit);

	        // Reference the shell as geometry instances and translate each instance accordingly.
	        // In this example we are building a 3D array of pyramids - 5x5x5.
	        int MAX_SHELLS_PER_ROW = 5;
	        int MAX_SHELLS_PER_COLUMN = 5;
	        int MAX_PAGE = 5;

	        model.GetSegmentKey().GetVisibilityControl().SetGenericEdges(true);
            HPS.SegmentKey refSegment = model.GetSegmentKey().Subsegment("reference geometries");

	        for (int page = 0; page < MAX_PAGE; page++)
	        {
		        for (int columns = 0; columns < MAX_SHELLS_PER_COLUMN; columns++)
		        {
			        for (int rows = 0; rows < MAX_SHELLS_PER_ROW; rows++)
			        {
				        HPS.MatrixKit matrixKit = new HPS.MatrixKit();

				        // Apply translation
				        matrixKit.Translate((float) rows, (float) columns, (float) page);
                        HPS.ReferenceKey refKey = refSegment.ReferenceGeometry(myShellKey);
				        refKey.SetModellingMatrix(matrixKit);
			        }
		        }
	        }

	        view.FitWorld();
	        view.GetSegmentKey().GetCameraControl().Orbit(45, 30);
	        canvas.UpdateWithNotifier().Wait();


            view.FitWorld().Update();
        }
    }
}
